==> two-minute-hate-0.5.tar.gz: two-minute-hate-0.5/world.c <==
/* $Id: world.c,v 1.53 2005/11/23 17:07:16 oohara Exp $ */

/* FILE */
#include <stdio.h>
/* malloc */
#include <stdlib.h>

#include <SDL.h>
/* libsdl-ttf2.0 */
#include <SDL_ttf.h>

#include "dbnj_util.h"
#include "dbnj_sdl.h"
#include "perlin.h"

#include "world.h"

static SDL_Surface **background_new(int n_errfile, FILE **p_errfile,
                                    const SDL_Surface *screen,
                                    int background_type);
static void draw_background_plain(int n_errfile, FILE **p_errfile,
                                  SDL_Surface *background, int i);
static void draw_background_perlin(int n_errfile, FILE **p_errfile,
                                   SDL_Surface *background, int i);
static int background_suffix_func(int x_size, int y_size, int x, int y);

/* return NULL on error */
world *
world_new(const char *ttf_font_file, int antialias, int anija,
          int full_screen, int background_type)
{
  int n_errfile = 0;
  FILE **p_errfile = NULL;
  world *new = NULL;

  /* sanity check */
  if (ttf_font_file == NULL)
  {
    fprintf(stderr, "world_new: ttf_font_file is NULL\n");
    return NULL;
  }

  p_errfile = (FILE **) malloc(sizeof(FILE *) * 1);
  if (p_errfile == NULL)
  {
    fprintf(stderr, "world_new: malloc(p_errfile) failed\n");
    return NULL;
  }
  n_errfile = 1;
  p_errfile[0] = stderr;

  new = (world *) malloc(sizeof(world));
  if (new == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: malloc(new) failed\n");
    free(p_errfile);
    n_errfile = 0;
    p_errfile = NULL;
    return NULL;
  }

  new->n_errfile = 0;
  new->p_errfile = NULL;
  new->screen = NULL;
  new->background = NULL;
  new->font_24 = NULL;
  new->font_12 = NULL;
  new->char_done = 0;
  new->char_done_cap = 0;
  new->miss_10 = 0;
  new->t_origin = 0;
  new->t_now = 0;
  new->frame = 0;
  new->clock_accurate = 0;
  new->antialias = 0;
  new->anija = 0;

  new->n_errfile = n_errfile;
  new->p_errfile = p_errfile;

  new->screen = dbnj_sdl_init(new->n_errfile, new->p_errfile,
                              480, 360, full_screen,
                              "two minute hate");
  if (new->screen == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: dbnj_sdl_init failed\n");
    world_delete(new);
    new = NULL;
    return NULL;
  }

  new->background = background_new(new->n_errfile, new->p_errfile,
                                   new->screen, background_type);
  if (new->background == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: background_new failed\n");
    world_delete(new);
    new = NULL;
    return NULL;
  }

  new->font_24 = dbnj_open_font(new->n_errfile, new->p_errfile,
                                ttf_font_file, 24);
  if (new->font_24 == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: dbnj_open_font(24) failed\n");
    world_delete(new);
    new = NULL;
    return NULL;
  }

  new->font_12 = dbnj_open_font(new->n_errfile, new->p_errfile,
                                ttf_font_file, 12);
  if (new->font_12 == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: dbnj_open_font(12) failed\n");
    world_delete(new);
    new = NULL;
    return NULL;
  }

  new->clock_accurate = dbnj_sdl_clock_accurate();
  new->antialias = antialias;
  new->anija = anija;

  return new;
}

void
world_delete(world *wp)
{
  int i;

  if (wp == NULL)
    return;

  if (wp->p_errfile != NULL)
  {
    for (i = 0; i < wp->n_errfile; i++)
      wp->p_errfile[i] = NULL;
    free(wp->p_errfile);
    wp->p_errfile = NULL;
  }
  if (wp->screen != NULL)
  {
    /* don't free the screen */
    wp->screen = NULL;
  }
  if (wp->background != NULL)
  {
    for (i = 0; i < 8; i++)
    {
      if (wp->background[i] != NULL)
      {  
        SDL_FreeSurface(wp->background[i]);
        wp->background[i] = NULL;
      }
    }
    free(wp->background);
    wp->background = NULL;
  }
  if (wp->font_24 != NULL)
  {
    TTF_CloseFont(wp->font_24);
    wp->font_24 = NULL;
  }
  if (wp->font_12 != NULL)
  {
    TTF_CloseFont(wp->font_12);
    wp->font_12 = NULL;
  }

  wp->n_errfile = 0;
  wp->p_errfile = NULL;
  wp->screen = NULL;
  wp->font_24 = NULL;
  wp->font_12 = NULL;
  wp->char_done = 0;
  wp->char_done_cap = 0;
  wp->miss_10 = 0;
  wp->t_origin = 0;
  wp->t_now = 0;
  wp->frame = 0;
  wp->clock_accurate = 0;
  wp->antialias = 0;
  wp->anija = 0;
}

/* return NULL on error */
static SDL_Surface **
background_new(int n_errfile, FILE **p_errfile,
               const SDL_Surface *screen, int background_type)
{
  int i;
  int j;
  SDL_Surface **new = NULL;
  SDL_Surface *temp = NULL;

  /* sanity check */
  if (screen == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "background_new: screen is NULL\n");
    return NULL;
  }

  new = (SDL_Surface **) malloc(sizeof(SDL_Surface *) * 8);
  if (new == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "background_new: malloc failed\n");
    return NULL;
  }

  for (i = 0; i < 8; i++)
    new[i] = NULL;

  for (i = 0; i < 8; i++)
  {
    new[i] = SDL_CreateRGBSurface(0, 480, 360, 32, 
                                  screen->format->Rmask,
                                  screen->format->Gmask,
                                  screen->format->Bmask,
                                  screen->format->Amask);
    if (new[i] == NULL)
    {
      dbnj_fprintf(n_errfile, p_errfile,
                   "background_new: SDL_CreateRGBSurface(%d) failed", i);
      if (SDL_GetError() != NULL)
        dbnj_fprintf(n_errfile, p_errfile,
                     " (%s)", SDL_GetError());
      dbnj_fprintf(n_errfile, p_errfile,
                   "\n");
      for (j = 0; j < 8; j++)
      {
        if (new[j] != NULL)
        {
          SDL_FreeSurface(new[j]);
          new[j] = NULL;
        }
      }
      free(new);
      new = NULL;
      return NULL;
    }

    temp = SDL_DisplayFormat(new[i]);
    if (temp == NULL)
    {
      dbnj_fprintf(n_errfile, p_errfile,
                   "background_new: SDL_DisplayFormat(%d) failed", i);
      if (SDL_GetError() != NULL)
        dbnj_fprintf(n_errfile, p_errfile,
                     " (%s)", SDL_GetError());
      dbnj_fprintf(n_errfile, p_errfile,
                   "\n");
      for (j = 0; j < 8; j++)
      {
        if (new[j] != NULL)
        {
          SDL_FreeSurface(new[j]);
          new[j] = NULL;
        }
      }
      free(new);
      new = NULL;
      return NULL;
    }
    new[i] = temp;

    if (background_type == 0)
      draw_background_plain(n_errfile, p_errfile,
                            new[i], i);
    else
      draw_background_perlin(n_errfile, p_errfile,
                             new[i], i);
  }

  return new;
}

static void draw_background_plain(int n_errfile, FILE **p_errfile,
                                  SDL_Surface *background, int i)
{
  int j;
  int k;
  int y;
  int hue;
  int value;
  int rgb[3];
  int rgb_from[3];
  int rgb_to[3];

  if (background == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "draw_background_plain: background is NULL\n");
    return;
  }

  hue = 210 + i * 7;
  value = 80;

  rgb_from[0] = 0;
  rgb_from[1] = 0;
  rgb_from[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_from, hue, 40, value);
  rgb_to[0] = 0;
  rgb_to[1] = 0;
  rgb_to[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_to, hue, 25, value);
  for (j = 0; j < 360; j++)
  {
    for (k = 0; k < 3; k++)
      rgb[k] = (rgb_from[k] * (360 - j) + rgb_to[k] * j) / 360;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, j, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }

  rgb_from[0] = 0;
  rgb_from[1] = 0;
  rgb_from[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_from, hue, 25, value);
  rgb_to[0] = 0;
  rgb_to[1] = 0;
  rgb_to[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_to, hue, 10, value);
  for (j = 0; j < i; j++)
  {
    y = 35 + j * 30;
    for (k = 0; k < 3; k++)
      rgb[k] = (rgb_from[k] * (360 - y) + rgb_to[k] * y) / 360;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, y, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }

  rgb_from[0] = 0;
  rgb_from[1] = 0;
  rgb_from[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_from, hue, 36, value);
  rgb_to[0] = 0;
  rgb_to[1] = 0;
  rgb_to[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_to, hue, 22, value);
  for (j = i; j < 7; j++)
  {
    y = 35 + j * 30;
    for (k = 0; k < 3; k++)
      rgb[k] = (rgb_from[k] * (360 - y) + rgb_to[k] * y) / 360;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, y, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }
}

static void
draw_background_perlin(int n_errfile, FILE **p_errfile,
                       SDL_Surface *background, int i)
{
  int j;
  int k;
  int n;
  int y;
  int hue;
  int saturation;
  int value;
  int rgb[3];
  int *noise = NULL;

  if (background == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "draw_background_perlin: background is NULL\n");
    return;
  }

  hue = 200 + i * 5;
  value = 75;

  noise = perlin_noise((int (*)(int, int, int, int)) &background_suffix_func,
                       480, 360, 5);
  if (noise == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "draw_background_perlin: noise is NULL\n");
    return;
  }

  n = 0;
  for (j = 0; j < 480; j++)
    for (k = 0; k < 360; k++)
      if (noise[background_suffix_func(480, 360, j, k)] >= 8)
        n++;

  for (j = 0; j < 480; j++)
    for (k = 0; k < 360; k++)
    {
      rgb[0] = 0;
      rgb[1] = 0;
      rgb[2] = 0;
      if (n < 480 * 360 / 2)
        saturation = 42 - noise[background_suffix_func(480, 360, j, k)] / 2;
      else
        saturation = 34 + noise[background_suffix_func(480, 360, j, k)] / 2;
      dbnj_hsv_to_rgb(n_errfile, p_errfile,
                      rgb, hue, saturation, value);
      dbnj_draw_rectangle(n_errfile, p_errfile,
                          background,
                          j, k, 1, 1,
                          rgb[0], rgb[1], rgb[2]);
    }

  rgb[0] = 0;
  rgb[1] = 0;
  rgb[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb, hue, 20, value);
  for (j = 0; j < i; j++)
  {
    y = 35 + j * 30;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, y, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }

  rgb[0] = 0;
  rgb[1] = 0;
  rgb[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb, hue, 42, value);
  for (j = i; j < 7; j++)
  {
    y = 35 + j * 30;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, y, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }

  free(noise);
  noise = NULL;
}

static int
background_suffix_func(int x_size, int y_size, int x, int y)
{
  return x * y_size + y;
}

==> two-minute-hate-0.6.tar.gz: two-minute-hate-0.6/world.c <==
/* $Id: world.c,v 1.68 2005/12/30 09:42:17 oohara Exp $ */

/* FILE */
#include <stdio.h>
/* malloc */
#include <stdlib.h>

#include <SDL.h>
/* libsdl-ttf2.0 */
#include <SDL_ttf.h>

#include "dbnj_util.h"
#include "dbnj_sdl.h"
#include "perlin.h"

#include "world.h"

static SDL_Surface **background_new(int n_errfile, FILE **p_errfile,
                                    const SDL_Surface *screen,
                                    int background_type);
static void draw_background_plain(int n_errfile, FILE **p_errfile,
                                  SDL_Surface *background, int i);
static void draw_background_perlin(int n_errfile, FILE **p_errfile,
                                   SDL_Surface *background, int i);
static int background_suffix_func(int x_size, int y_size, int x, int y);

/* return NULL on error */
world *
world_new(const char *ttf_font_file, int antialias, int anija,
          int full_screen, int background_type)
{
  int n_errfile = 0;
  FILE **p_errfile = NULL;
  world *new = NULL;

  /* sanity check */
  if (ttf_font_file == NULL)
  {
    fprintf(stderr, "world_new: ttf_font_file is NULL\n");
    return NULL;
  }

  p_errfile = (FILE **) malloc(sizeof(FILE *) * 1);
  if (p_errfile == NULL)
  {
    fprintf(stderr, "world_new: malloc(p_errfile) failed\n");
    return NULL;
  }
  n_errfile = 1;
  p_errfile[0] = stderr;

  new = (world *) malloc(sizeof(world));
  if (new == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: malloc(new) failed\n");
    free(p_errfile);
    n_errfile = 0;
    p_errfile = NULL;
    return NULL;
  }

  new->n_errfile = 0;
  new->p_errfile = NULL;
  new->screen = NULL;
  new->background = NULL;
  new->font_24 = NULL;
  new->font_12 = NULL;
  new->char_done = 0;
  new->char_done_cap = 0;
  new->miss_10 = 0;
  new->t_origin = 0;
  new->t_now = 0;
  new->frame = 0;
  new->clock_accurate = 0;
  new->antialias = 0;
  new->anija = 0;

  new->n_errfile = n_errfile;
  new->p_errfile = p_errfile;

  new->screen = dbnj_sdl_init(new->n_errfile, new->p_errfile,
                              480, 360, full_screen,
                              "two minute hate");
  if (new->screen == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: dbnj_sdl_init failed\n");
    world_delete(new);
    new = NULL;
    return NULL;
  }

  new->background = background_new(new->n_errfile, new->p_errfile,
                                   new->screen, background_type);
  if (new->background == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: background_new failed\n");
    world_delete(new);
    new = NULL;
    return NULL;
  }

  new->font_24 = dbnj_open_font(new->n_errfile, new->p_errfile,
                                ttf_font_file, 24);
  if (new->font_24 == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: dbnj_open_font(24) failed\n");
    world_delete(new);
    new = NULL;
    return NULL;
  }

  new->font_12 = dbnj_open_font(new->n_errfile, new->p_errfile,
                                ttf_font_file, 12);
  if (new->font_12 == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "world_new: dbnj_open_font(12) failed\n");
    world_delete(new);
    new = NULL;
    return NULL;
  }

  new->clock_accurate = dbnj_sdl_clock_accurate();
  new->antialias = antialias;
  new->anija = anija;

  return new;
}

void
world_delete(world *wp)
{
  int i;

  if (wp == NULL)
    return;

  if (wp->p_errfile != NULL)
  {
    for (i = 0; i < wp->n_errfile; i++)
      wp->p_errfile[i] = NULL;
    free(wp->p_errfile);
    wp->p_errfile = NULL;
  }
  if (wp->screen != NULL)
  {
    /* don't free the screen */
    wp->screen = NULL;
  }
  if (wp->background != NULL)
  {
    for (i = 0; i < 8; i++)
    {
      if (wp->background[i] != NULL)
      {  
        SDL_FreeSurface(wp->background[i]);
        wp->background[i] = NULL;
      }
    }
    free(wp->background);
    wp->background = NULL;
  }
  if (wp->font_24 != NULL)
  {
    TTF_CloseFont(wp->font_24);
    wp->font_24 = NULL;
  }
  if (wp->font_12 != NULL)
  {
    TTF_CloseFont(wp->font_12);
    wp->font_12 = NULL;
  }

  wp->n_errfile = 0;
  wp->p_errfile = NULL;
  wp->screen = NULL;
  wp->font_24 = NULL;
  wp->font_12 = NULL;
  wp->char_done = 0;
  wp->char_done_cap = 0;
  wp->miss_10 = 0;
  wp->t_origin = 0;
  wp->t_now = 0;
  wp->frame = 0;
  wp->clock_accurate = 0;
  wp->antialias = 0;
  wp->anija = 0;
}

/* return NULL on error */
static SDL_Surface **
background_new(int n_errfile, FILE **p_errfile,
               const SDL_Surface *screen, int background_type)
{
  int i;
  int j;
  SDL_Surface **new = NULL;
  SDL_Surface *temp = NULL;

  /* sanity check */
  if (screen == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "background_new: screen is NULL\n");
    return NULL;
  }

  new = (SDL_Surface **) malloc(sizeof(SDL_Surface *) * 8);
  if (new == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "background_new: malloc failed\n");
    return NULL;
  }

  for (i = 0; i < 8; i++)
    new[i] = NULL;

  for (i = 0; i < 8; i++)
  {
    new[i] = SDL_CreateRGBSurface(0, 480, 360, 32, 
                                  screen->format->Rmask,
                                  screen->format->Gmask,
                                  screen->format->Bmask,
                                  screen->format->Amask);
    if (new[i] == NULL)
    {
      dbnj_fprintf(n_errfile, p_errfile,
                   "background_new: SDL_CreateRGBSurface(%d) failed", i);
      if (SDL_GetError() != NULL)
        dbnj_fprintf(n_errfile, p_errfile,
                     " (%s)", SDL_GetError());
      dbnj_fprintf(n_errfile, p_errfile,
                   "\n");
      for (j = 0; j < 8; j++)
      {
        if (new[j] != NULL)
        {
          SDL_FreeSurface(new[j]);
          new[j] = NULL;
        }
      }
      free(new);
      new = NULL;
      return NULL;
    }

    temp = SDL_DisplayFormat(new[i]);
    if (temp == NULL)
    {
      dbnj_fprintf(n_errfile, p_errfile,
                   "background_new: SDL_DisplayFormat(%d) failed", i);
      if (SDL_GetError() != NULL)
        dbnj_fprintf(n_errfile, p_errfile,
                     " (%s)", SDL_GetError());
      dbnj_fprintf(n_errfile, p_errfile,
                   "\n");
      for (j = 0; j < 8; j++)
      {
        if (new[j] != NULL)
        {
          SDL_FreeSurface(new[j]);
          new[j] = NULL;
        }
      }
      free(new);
      new = NULL;
      return NULL;
    }
    new[i] = temp;

    if (background_type == 0)
      draw_background_plain(n_errfile, p_errfile,
                            new[i], i);
    else
      draw_background_perlin(n_errfile, p_errfile,
                             new[i], i);
  }

  return new;
}

static void draw_background_plain(int n_errfile, FILE **p_errfile,
                                  SDL_Surface *background, int i)
{
  int j;
  int k;
  int y;
  int hue;
  int value;
  int rgb[3];
  int rgb_from[3];
  int rgb_to[3];

  if (background == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "draw_background_plain: background is NULL\n");
    return;
  }

  hue = 210 + i * 7;
  value = 80;

  rgb_from[0] = 0;
  rgb_from[1] = 0;
  rgb_from[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_from, hue, 40, value);
  rgb_to[0] = 0;
  rgb_to[1] = 0;
  rgb_to[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_to, hue, 25, value);
  for (j = 0; j < 360; j++)
  {
    for (k = 0; k < 3; k++)
      rgb[k] = (rgb_from[k] * (360 - j) + rgb_to[k] * j) / 360;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, j, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }

  rgb_from[0] = 0;
  rgb_from[1] = 0;
  rgb_from[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_from, hue, 25, value);
  rgb_to[0] = 0;
  rgb_to[1] = 0;
  rgb_to[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_to, hue, 10, value);
  for (j = 0; j < i; j++)
  {
    y = 35 + j * 30;
    for (k = 0; k < 3; k++)
      rgb[k] = (rgb_from[k] * (360 - y) + rgb_to[k] * y) / 360;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, y, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }

  rgb_from[0] = 0;
  rgb_from[1] = 0;
  rgb_from[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_from, hue, 36, value);
  rgb_to[0] = 0;
  rgb_to[1] = 0;
  rgb_to[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_to, hue, 22, value);
  for (j = i; j < 7; j++)
  {
    y = 35 + j * 30;
    for (k = 0; k < 3; k++)
      rgb[k] = (rgb_from[k] * (360 - y) + rgb_to[k] * y) / 360;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, y, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }
}

static void
draw_background_perlin(int n_errfile, FILE **p_errfile,
                       SDL_Surface *background, int i)
{
  int j;
  int k;
  int m;
  int c;
  int n;
  int y;
  int hue;
  int value;
  int rgb_from[3];
  int rgb_to[3];
  int rgb_between[48];
  int rgb_soft[48];
  int rgb[3];
  int *noise = NULL;

  if (background == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "draw_background_perlin: background is NULL\n");
    return;
  }

  hue = 200 + i * 5;
  value = 65;

  noise = perlin_noise((int (*)(int, int, int, int)) &background_suffix_func,
                       480, 360, 5);
  if (noise == NULL)
  {
    dbnj_fprintf(n_errfile, p_errfile,
                 "draw_background_perlin: noise is NULL\n");
    return;
  }

  n = 0;
  for (j = 0; j < 480; j++)
    for (k = 0; k < 360; k++)
      if (noise[background_suffix_func(480, 360, j, k)] >= 8)
        n++;

  for (m = 0; m < 3; m++)
  {
    rgb_from[m] = 0;
    rgb_to[m] = 0;
  }
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_from, hue, 30, value + 5);
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_to, hue, 45, value + 5);
  for (c = 0; c < 16; c++)
    for (m = 0; m < 3; m++)
    {
      rgb_between[c * 3 + m] = (rgb_from[m] * (15 - c) + rgb_to[m] * c) / 15;
    }

  for (m = 0; m < 3; m++)
  {
    rgb_from[m] = 0;
    rgb_to[m] = 0;
  }
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_from, hue, 38, value);
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb_to, hue, 45, value);
  for (c = 0; c < 16; c++)
    for (m = 0; m < 3; m++)
    {
      rgb_soft[c * 3 + m] = (rgb_from[m] * (15 - c) + rgb_to[m] * c) / 15;
    }

  for (j = 0; j < 480; j++)
    for (k = 0; k < 360; k++)
    {
      c = noise[background_suffix_func(480, 360, j, k)];
      if (n < 480 * 360 / 2)
        c = 15 - c;
      if (c < 3)
        c = 3;
      if (c > 12)
        c = 12;
      for (m = 0; m < 3; m++)
        rgb[m] = rgb_between[c * 3 + m];
      if (k < 10)
      {
        for (m = 0; m < 3; m++)
        {    
          rgb[m] = rgb_soft[c * 3 + m] * k;
          rgb[m] += rgb_between[c * 3 + m] * (10 - k);
          rgb[m] /= 10;
        }
      }
      else if (k < 10 + 30 * 7)
      {
        for (m = 0; m < 3; m++)
          rgb[m] = rgb_soft[c * 3 + m];
      }
      else if (k < 10 + 30 * 7 + 10)
      {
        for (m = 0; m < 3; m++)
        {    
          rgb[m] = rgb_soft[c * 3 + m] * (10 - (k - (10 + 30 * 7)));
          rgb[m] += rgb_between[c * 3 + m] * (k - (10 + 30 * 7));
          rgb[m] /= 10;
        }
      }

      dbnj_draw_rectangle(n_errfile, p_errfile,
                          background,
                          j, k, 1, 1,
                          rgb[0], rgb[1], rgb[2]);
    }

  rgb[0] = 0;
  rgb[1] = 0;
  rgb[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb, hue, 20, value + 5);
  for (j = 0; j < i; j++)
  {
    y = 35 + j * 30;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, y, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }

  rgb[0] = 0;
  rgb[1] = 0;
  rgb[2] = 0;
  dbnj_hsv_to_rgb(n_errfile, p_errfile,
                  rgb, hue, 45, value);
  for (j = i; j < 7; j++)
  {
    y = 35 + j * 30;
    dbnj_draw_rectangle(n_errfile, p_errfile,
                        background,
                        0, y, 480, 1,
                        rgb[0], rgb[1], rgb[2]);
  }

  free(noise);
  noise = NULL;
}

static int
background_suffix_func(int x_size, int y_size, int x, int y)
{
  return x * y_size + y;
}
